
#include <windows.h>  
#include "printdlg.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "PrintDlg()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

LPCTSTR lpszText = "This sample text will be printed to the printer.";

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static PAGESETUPDLG psd;
static HWND         hEdit = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              psd.lStructSize     = sizeof( PAGESETUPDLG );
              psd.hwndOwner       = hWnd;
              psd.rtMargin.top    = 1000;
              psd.rtMargin.left   = 1000;
              psd.rtMargin.bottom = 1000;
              psd.rtMargin.right  = 1000;  
              psd.hDevMode        = GlobalAlloc( GHND, sizeof( DEVMODE ) );
              psd.hDevNames       = GlobalAlloc( GHND, sizeof( DEVNAMES ) );
              psd.Flags           = PSD_MARGINS | PSD_INTHOUSANDTHSOFINCHES; 

              hEdit = CreateWindow( "EDIT", "",    
                                    WS_CHILD | ES_LEFT | 
                                    WS_VISIBLE | ES_MULTILINE | ES_NOHIDESEL, 
                                    0, 0, 
                                    0, 0,  
                                    hWnd,              
                                    (HMENU)101,              
                                    hInst,         
                                    NULL               
                                  );

              SendMessage( hEdit, WM_SETTEXT, 0, (LPARAM)lpszText );
              break;

      case WM_SIZE :
              MoveWindow( hEdit, 0, 0, LOWORD( lParam ), HIWORD( lParam ), TRUE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_PAGESETUP :
                        PageSetupDlg( &psd );
                        break;

                 case IDM_PRINT :
                        {
                           PRINTDLG pd;

                           // Initialize the PRINTDLG structure.
                           //...................................
                           memset( &pd, 0, sizeof( PRINTDLG ) );
                           pd.lStructSize    = sizeof( PRINTDLG );
                           pd.hwndOwner      = hWnd;
                           pd.Flags          = PD_RETURNDC;
                           pd.hDevMode       = psd.hDevMode;
                           pd.hDevNames      = psd.hDevNames;

                           // Create the print dialog.
                           //.........................
                           if ( PrintDlg( &pd ) )
                           {
                              WORD      wCopies;
                              char      szBuf[512];
                              RECT      rect;
                              DOCINFO   di;
                              LPDEVMODE lpDevMode = (LPDEVMODE)GlobalLock( pd.hDevMode );

                              memset( &di, 0, sizeof( DOCINFO ) );

                              // Initialize rect to size of page.
                              //.................................
                              rect.top = 
                                     ( GetDeviceCaps( pd.hDC, LOGPIXELSY )*
                                     (psd.rtMargin.top/1000 ))-
                                     GetDeviceCaps( pd.hDC, PHYSICALOFFSETY );

                              rect.left = 
                                     ( GetDeviceCaps( pd.hDC, LOGPIXELSX )*
                                     (psd.rtMargin.left/1000 ))-
                                     GetDeviceCaps( pd.hDC, PHYSICALOFFSETX );

                              rect.right = 
                                     lpDevMode->dmPelsWidth - 
                                     ( GetDeviceCaps( pd.hDC, LOGPIXELSX )*
                                     (psd.rtMargin.right/1000 ))-
                                     GetDeviceCaps( pd.hDC, PHYSICALOFFSETX );

                              rect.bottom = 
                                     lpDevMode->dmPelsHeight - 
                                     ( GetDeviceCaps( pd.hDC, LOGPIXELSY )*
                                     (psd.rtMargin.bottom/1000 ))-
                                     GetDeviceCaps( pd.hDC, PHYSICALOFFSETY );

                              // Initialize di with the document title.
                              //.......................................
                              di.cbSize      = sizeof( DOCINFO );
                              di.lpszDocName = lpszTitle;

                              // Retrieve the text from the edit control.
                              //.........................................
                              SendMessage( hEdit, WM_GETTEXT, sizeof( szBuf ), (LPARAM)szBuf );

                              // Print the text.
                              //................
                              if ( StartDoc( pd.hDC, &di ) > 0 )
                              {
                                 for( wCopies = 0; wCopies < pd.nCopies; wCopies++ )
                                 {
                                    StartPage( pd.hDC );
                                    DrawText( pd.hDC, szBuf, strlen( szBuf ), &rect, DT_LEFT | DT_WORDBREAK );
                                    EndPage( pd.hDC );
                                 }

                                 EndDoc( pd.hDC );
                              }

                              // Unlock and free resources.
                              //...........................
                              GlobalUnlock( pd.hDevMode );
                              DeleteDC( pd.hDC ); 
                           }
                           else if ( CommDlgExtendedError() == PDERR_NODEFAULTPRN )
                              MessageBox( hWnd, "No default printer selected.", 
                                                "Print", MB_OK | MB_ICONSTOP );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;

      case WM_DESTROY :
              GlobalFree( psd.hDevMode );
              GlobalFree( psd.hDevNames );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
